@@ -2,7 +2,7 @@ use error_chain::error_chain;
 use std::collections::{HashMap, HashSet};
 use std::ffi::{OsStr, OsString};
 use std::fs::{self, File, OpenOptions};
-use std::io::{copy, BufReader, Seek, SeekFrom, Write};
+use std::io::{copy, BufReader, Read, Seek, SeekFrom, Write};
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 
@@ -28,10 +28,10 @@ async fn main() -> Result<()> {
         .arg(
             Arg::with_name("verbose")
                 .long("verbose")
-                .default_value("error")
+                .default_value("trace")
                 .takes_value(true)
                 .long_help(
-                    "Set the level of verbosity (off, error (default), warn, info, debug, trace)",
+                    "Set the level of verbosity for the local minidum-stackwalk (off, error, warn, info, debug, trace (default))",
                 ),
         )
         .arg(
@@ -42,7 +42,9 @@ async fn main() -> Result<()> {
                 .long_help(
                     "Fields in the json output to ignore the value of during comparison.
 
-If no values are provided, the default ignores will be:
+This is additive to the default ignore list. If you wish to disable the defaults, use --no-default-ignores
+
+The default ignores are:
 
 deprecated fields:
   * frames_truncated
@@ -54,6 +56,7 @@ redundant fields:
   * total_frames
   * frame_count
   * threads_index
+  * frame
 
 debugging fields:
   * missing_symbols
@@ -67,6 +70,13 @@ which you don't care about the value of, or fields which contain redundant data
 the length of an array.\n\n\n",
                 ),
         )
+        .arg(
+            Arg::with_name("no-default-ignores")
+                .long("no-default-ignores")
+                .long_help(
+                    "Disable the default list of --ignore-field.\n\n",
+                ),
+        )
         .arg(
             Arg::with_name("compare")
                 .long("compare")
@@ -154,8 +164,7 @@ To this end, the default value for this flag is a `rust-minidump-cache` \
 subdirectory of `std::env::temp_dir()` (usually /tmp/rust-minidump-cache on linux).
 
 symbols-cache must be on the same filesystem as symbols-tmp (if that doesn't mean anything to \
-you, don't worry about it, you're probably not doing something that will run afoul of it).
-\n\n\n",
+you, don't worry about it, you're probably not doing something that will run afoul of it).\n\n\n",
                 ),
         )
         .arg(
@@ -174,8 +183,7 @@ See the rust documentation for how to set that value if you wish to use somethin
 your system's default temp directory.
 
 symbols-tmp must be on the same filesystem as symbols-cache (if that doesn't mean anything to \
-you, don't worry about it, you're probably not doing something that will run afoul of it).
-\n\n\n",
+you, don't worry about it, you're probably not doing something that will run afoul of it).\n\n\n",
                 ),
         )
         .arg(
@@ -197,9 +205,7 @@ you, don't worry about it, you're probably not doing something that will run afo
                 .long_help(
                     "A path to a local rust-minidump/minidump-stackwalk checkout to build and run
 
-`cargo run --release -- <args>` will be invoked in the given directory.
-
-",
+`cargo run --release -- <args>` will be invoked in the given directory.\n\n\n",
                 ),
         )
         .arg(
@@ -215,8 +221,7 @@ Required permissions:
 * 'View Raw Dumps' (hard required)
 * 'View Personal Identifiable Information' (to read some entries in raw-json)
 
-See https://crash-stats.mozilla.org/api/tokens/ for details.
-\n\n\n",
+See https://crash-stats.mozilla.org/api/tokens/ for details.\n\n\n",
                 ),
         )
         .arg(
@@ -225,16 +230,9 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.
                 .takes_value(true)
                 .required(true)
                 .long_help(
-                    "The socorro crash id to analyze
-\n\n\n",
+                    "The socorro crash id to analyze.\n\n",
                 ),
         )
-        .after_help(
-            "
-
-
-",
-        )
         .get_matches();
 
     let output_file = matches
@@ -249,14 +247,10 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.
         .value_of_os("run-local")
         .map(|os_str| Path::new(os_str).to_owned());
 
-    let verbosity = match matches.value_of("verbose").unwrap() {
-        "off" => LevelFilter::Off,
-        "warn" => LevelFilter::Warn,
-        "info" => LevelFilter::Info,
-        "debug" => LevelFilter::Debug,
-        "trace" => LevelFilter::Trace,
-        _ => LevelFilter::Error,
-    };
+    let verbose = matches.value_of("verbose").unwrap();
+    // Verbose configures the child minidump-stackwalk, this app
+    // doesn't have significant logging.
+    let verbosity = LevelFilter::Warn;
 
     let mut stdout;
     let mut output_f;
@@ -338,6 +332,7 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.
         "total_frames",
         "frame_count",
         "threads_index",
+        "frame",
         // execution debugging
         "missing_symbols",
         "symbol_disk_cache_hit",
@@ -346,17 +341,18 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.
         "symbol_fetch_time",
     ];
 
-    let ignored_fields = matches
-        .values_of("ignore-field")
-        .map(|s| s.collect::<Vec<_>>());
+    let no_default_ignores = matches.is_present("no-default-ignores");
 
-    let ignored_fields = ignored_fields
-        .as_ref()
-        .map(|v| v.as_slice())
-        .unwrap_or(&default_ignored_fields)
-        .iter()
-        .copied()
-        .collect::<HashSet<&str>>();
+    let mut ignored_fields = HashSet::new();
+    if !no_default_ignores {
+        ignored_fields.extend(default_ignored_fields);
+    }
+    ignored_fields.extend(
+        matches
+            .values_of("ignore-field")
+            .unwrap_or_default()
+            .into_iter(),
+    );
 
     let compare_field = matches.value_of("compare");
 
@@ -479,30 +475,50 @@ See https://crash-stats.mozilla.org/api/tokens/ for details.
         command = command.arg(arg);
     }
 
-    command = command.arg(&minidump).arg("--verbose=trace");
+    {
+        let mut arg = OsString::from("--verbose=");
+        arg.push(&verbose);
+        command = command.arg(arg);
+    }
+
+    command = command.arg(&minidump);
 
     // Different approaches to forwarding subprocess stdout based on
     // whether we're writing to a file or not.
-    if output_file.is_some() {
+    let status = if output_file.is_some() {
         let output = command
             .stdout(Stdio::piped())
             .stderr(Stdio::inherit())
-            .output()
-            .unwrap();
+            .output()?;
+
         writeln!(f, "{}", String::from_utf8_lossy(&output.stdout))?;
+        output.status
     } else {
-        let _ = command.status().unwrap();
+        command.status()?
+    };
+
+    if status.success() {
+        writeln!(f, "done!")?;
+        writeln!(f, "comparing json...")?;
+        let rust_json_file = File::open(&local_output)?;
+        let rust_json: serde_json::Value =
+            serde_json::from_reader(BufReader::new(rust_json_file)).unwrap();
+
+        compare_crashes(f, compare_field, &ignored_fields, &socc_json, &rust_json)?;
+    } else {
+        if let Some(code) = status.code() {
+            writeln!(f, "failed! exit status: {}", code)?;
+        } else {
+            writeln!(f, "failed! (no exit status, terminated by signal?)")?;
+        }
+        writeln!(f, "dumping logs: ")?;
+
+        let rust_log_file = File::open(&local_logs)?;
+        let mut logs = String::new();
+        BufReader::new(rust_log_file).read_to_string(&mut logs)?;
+        writeln!(f, "{}", logs)?;
     }
 
-    writeln!(f, "done!")?;
-    writeln!(f, "comparing json...")?;
-
-    let rust_json_file = File::open(&local_output).unwrap();
-    let rust_json: serde_json::Value =
-        serde_json::from_reader(BufReader::new(rust_json_file)).unwrap();
-
-    compare_crashes(f, compare_field, &ignored_fields, &socc_json, &rust_json)?;
-
     writeln!(f, "\nOutput Files: ")?;
     writeln!(f, "  * Minidump: {}", minidump.display())?;
     writeln!(f, "  * Socorro Processed Crash: {}", socc_output.display())?;
@@ -616,6 +632,16 @@ fn recursive_compare(
             if let (Some(s), Some(r)) = (parse_int(s), parse_int(r)) {
                 if s == r {
                     writeln!(f, " {:width$}{}: 0x{:08x}", "", k, s, width = depth)?;
+                } else if ignored.contains(&k) {
+                    writeln!(
+                        f,
+                        "~{:width$}ignoring field {}: 0x{:08x}",
+                        "",
+                        k,
+                        s,
+                        width = depth
+                    )?;
+                    warnings += 1;
                 } else {
                     errors += 1;
                     writeln!(f, "-{:width$}did not match", "", width = depth)?;
@@ -625,6 +651,15 @@ fn recursive_compare(
             } else {
                 if s == r {
                     writeln!(f, " {:width$}{}: {}", "", k, s, width = depth)?;
+                } else if ignored.contains(&k) {
+                    writeln!(
+                        f,
+                        "~{:width$}ignoring field {}: {}",
+                        "",
+                        k,
+                        s,
+                        width = depth
+                    )?;
                 } else if k == "trust" {
                     let (s_trust, r_trust) = trust_levels(socc_val, rust_val);
                     if r_trust < s_trust {
@@ -723,7 +758,7 @@ fn recursive_compare(
                     errors += new_errors;
                     warnings += new_warnings;
                 }
-                writeln!(f, "{:width$} {}: ]", "", k, width = depth)?;
+                writeln!(f, "{:width$} ]", "", width = depth)?;
             } else {
                 // The bulk of the refined analysis happens here, as we try to more intelligently
                 // handle the array of frames in a backtrace. This is important because very small
@@ -753,9 +788,9 @@ fn recursive_compare(
                     // a few frames will get everything back in sync.
                     if let (Object(s_obj), Object(r_obj)) = (s_val, r_val) {
                         if let (Some(String(s_func)), Some(String(r_func))) =
-                            (s_obj.get("function"), r_obj.get("function"))
+                            (s_obj.get("offset"), r_obj.get("offset"))
                         {
-                            if s_func != r_func {
+                            if parse_int(s_func) != parse_int(r_func) {
                                 are_different_frames = true;
                                 // Assume one of the values is "good" and scan ahead in the other for a match
                                 let try_lookahead =
@@ -771,9 +806,9 @@ fn recursive_compare(
                                                 if let (
                                                     Some(String(good_func)),
                                                     Some(String(bad_func)),
-                                                ) = (good.get("function"), bad.get("function"))
+                                                ) = (good.get("offset"), bad.get("offset"))
                                                 {
-                                                    if good_func == bad_func {
+                                                    if parse_int(good_func) == parse_int(bad_func) {
                                                         return Some(i);
                                                     }
                                                 }
@@ -947,12 +982,12 @@ fn recursive_compare(
             }
         }
         (Null, Null) => {
-            writeln!(f, " {:width$}{}: null", "", k, width=depth)?;
+            writeln!(f, " {:width$}{}: null", "", k, width = depth)?;
         }
         (_, Null) => {
             if socc_val.as_str() == Some("") {
                 // Socorro sometimes has blanks for nulls, consider them equal
-                writeln!(f, " {:width$}{}: null", "", k, width=depth)?;
+                writeln!(f, " {:width$}{}: null", "", k, width = depth)?;
             } else {
                 if ignored.contains(k) {
                     warnings += 1;
